import 'package:stateful_provider/process_error.dart';
import 'package:uuid/uuid.dart';

import 'task_provider.dart';

class ProcessStatus extends TaskProvider {
  factory ProcessStatus() => _singleton;

  static final ProcessStatus _singleton = ProcessStatus._internal();

  ProcessStatus._internal();

  Set<ProcessError> _exceptions = {};

  ProcessError get error => _exceptions.isEmpty ? null : _exceptions.first;

  void add(Uuid uuid) {
    processes.add(uuid);
    _rebuildIfNeeded();
  }

  void complete(Uuid uuid) {
    processes.remove(uuid);
    _rebuildIfNeeded();
  }

  bool _isProcessing = false;

  void _rebuildIfNeeded() {
    final current = isProcessing;
    if (current == _isProcessing) return;
    _isProcessing = current;
    notifyListeners();
  }

  void catchError(ProcessError error) {
    _exceptions.add(error);
    notifyListeners();
  }

  void consumeError(ProcessError error) {
    _exceptions.remove(error);
    notifyListeners();
  }
}
